"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .unifiedid import UnifiedID, UnifiedIDTypedDict
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CreateBankAccountResponseTypedDict(TypedDict):
    r"""Bank Account created"""

    status_code: int
    r"""HTTP Response Status Code"""
    status: str
    r"""HTTP Response Status"""
    data: UnifiedIDTypedDict
    r"""A object containing a unique identifier for the resource that was created, updated, or deleted."""
    service: NotRequired[str]
    r"""Apideck ID of service provider"""
    resource: NotRequired[str]
    r"""Unified API resource name"""
    operation: NotRequired[str]
    r"""Operation performed"""


class CreateBankAccountResponse(BaseModel):
    r"""Bank Account created"""

    status_code: int
    r"""HTTP Response Status Code"""

    status: str
    r"""HTTP Response Status"""

    data: UnifiedID
    r"""A object containing a unique identifier for the resource that was created, updated, or deleted."""

    service: Optional[str] = None
    r"""Apideck ID of service provider"""

    resource: Optional[str] = None
    r"""Unified API resource name"""

    operation: Optional[str] = None
    r"""Operation performed"""
