"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .datascopesresources import DataScopesResources, DataScopesResourcesTypedDict
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DataScopesInputTypedDict(TypedDict):
    enabled: NotRequired[bool]
    r"""Whether Data Scopes is enabled for this application"""
    resources: NotRequired[DataScopesResourcesTypedDict]
    r"""Data scopes resource configuration that can be either detailed field permissions or a wildcard"""


class DataScopesInput(BaseModel):
    enabled: Optional[bool] = None
    r"""Whether Data Scopes is enabled for this application"""

    resources: Optional[DataScopesResources] = None
    r"""Data scopes resource configuration that can be either detailed field permissions or a wildcard"""
