"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .datascopesresources import DataScopesResources, DataScopesResourcesTypedDict
from apideck_unify.types import BaseModel
from typing_extensions import TypedDict


class ConsentRecordInputTypedDict(TypedDict):
    granted: bool
    r"""Whether consent was granted (true) or denied/revoked (false)"""
    resources: DataScopesResourcesTypedDict
    r"""Data scopes resource configuration that can be either detailed field permissions or a wildcard"""


class ConsentRecordInput(BaseModel):
    granted: bool
    r"""Whether consent was granted (true) or denied/revoked (false)"""

    resources: DataScopesResources
    r"""Data scopes resource configuration that can be either detailed field permissions or a wildcard"""
