"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ConsentState(str, Enum):
    r"""The current consent state of the connection"""

    IMPLICIT = "implicit"
    PENDING = "pending"
    GRANTED = "granted"
    DENIED = "denied"
    REVOKED = "revoked"
    REQUIRES_RECONSENT = "requires_reconsent"
